// ===============================================
// File:		      GLsettings.cpp
// Description:		
// Version 1:
// Current:
// Author(s):		Thomas Kern
// ===============================================

/*
 * Copyright (C) 1997-1998  Thomas Kern
 */

// 

//=====================================================================
//	INCLUDES
//=====================================================================
//
#include <math.h>
#include <stdio.h>

#ifdef _WIN32
 #include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>

#include "GLsettings.H"

//=====================================================================
//	STEUERUNG DES OPENGL-STATUS
//=====================================================================
//
// Konstruktor
OpenGLState::OpenGLState(int winWidth, int winHeight)
{ 
  // die Groesse des Fensters ist bereits vorgegeben
  initWidth  = winWidth;
  initHeight = winHeight;
  
  Reset();
}

// --------------------------------------------------------------------
// Destruktor
OpenGLState::~OpenGLState()
{

}

// --------------------------------------------------------------------
// Festlegen der Startwerte, wird durch die Config-datei ueberschrieben 
void OpenGLState::Reset()
{
  initAngle     =  60.0; // degrees
  if(initHeight)
    initAspect  = (GLfloat)initWidth / (GLfloat)initHeight;  
  else
    initAspect  =   1.0; // height = widht
  initNearPlane =   0.1;
  initFarPlane  = 500.0;

  // andere Vorgabewerte aus einer Datei einlesen
  // InitFromFile();

  // nun diese Initialisierungen als (veraenderliche) Startwerte nutzen
   CopyInitValues();
}

// --------------------------------------------------------------------
// Initialisierung 
void OpenGLState::InitFromFile()
{
  
}

// --------------------------------------------------------------------
// Initialisierung 
void OpenGLState::CopyInitValues()
{
  currentWidth  = initWidth;
  currentHeight = initHeight;
  ViewingAngle  = initAngle;
  WindowAspect  = initAspect;
  nearPlane     = initNearPlane;
  farPlane      = initFarPlane;
}

// --------------------------------------------------------------------
// Perspektive korrigieren
// ViewingAngle: field of view angle(degrees), in y direction.
// WindowAspect: aspect ratio = field of view in x direction. 
// The aspect ratio is the ratio of x (width) to y (height).
void OpenGLState::SetPerspective(int newWidth, int newHeight)
{ 
  // 1: die uebergebenen Werte kopieren  
  if(newWidth)
    currentWidth  = newWidth;
  if(newHeight)
    currentHeight = newHeight; // TypeCast fuer Berechnungen

  // 2: Berechnungen zur Neufestlegung durchfuehren
  WindowAspect = (GLfloat)currentWidth / (GLfloat)currentHeight;

  // 3: die Projektionsmatrix veraendern
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  // 4: das Blickfeld neu festlegen
  gluPerspective(ViewingAngle, WindowAspect, nearPlane, farPlane);

  // 5: alle folgenden Operationen beeinflussen die Modellmatrix
  glMatrixMode(GL_MODELVIEW);

  // 6: Viewport neu festlegen
  glViewport(0, 0, currentWidth, currentHeight);
}

// --------------------------------------------------------------------
// der Betrachter steht im Ursprung
void OpenGLState::ClearEyeMovement()
{
  int i, j;
  eyeMoveCount = 0;
  for(i = 0; i < MAX_TRANSFORMS; i++) {
    nextMove[i][j] = (GLfloat)skip;
    for(j = 1; j < 5; j++)
      nextMove[i][j] = 0.0;    
  }
}

// --------------------------------------------------------------------
// der Betrachter bewegt sich von seiner Position weg
void OpenGLState::SetEyeTransformation(int pos, GLfloat point[3])
{
  // ueber pos wird die Reihenfolge der Bewegungen gesteuert
  if(pos && pos < MAX_TRANSFORMS) {
    int i;
    nextMove[pos][0] = (GLfloat)trans; // Transformation
    for(i = 0; i < 3; i++)
      nextMove[pos][i+1] = point[i];   // wohin
    if(eyeMoveCount < pos)             // Anzahl der benoetigten
      eyeMoveCount = pos;              // OpenGL Aufrufe
  }
}

// --------------------------------------------------------------------
// der Betrachter dreht sich um seine Position
void OpenGLState::SetEyeRotation(int pos, GLfloat angle, GLfloat vector[3])
{
  // ueber pos wird die Reihenfolge der Bewegungen gesteuert
  if(pos && pos < MAX_TRANSFORMS) {
    int i;
    nextMove[pos][0] = (GLfloat)rot;   // Rotation
    nextMove[pos][1] = angle;          // Winkel
    for(i = 0; i < 3; i++)
      nextMove[pos][i+2] = vector[i];  // Achse
    if(eyeMoveCount < pos)             // Anzahl der benoetigten
      eyeMoveCount = pos;              // OpenGL Aufrufe
  }
}

